==> observable-redux-json-api-0.0.9.tgz: package/src/jsonapi.ts <==
import { __assign } from "./assign"; // tslint:disable-line

import { createAction, handleActions } from "redux-actions";
import actionTypes from "./constants";
import { Observable } from "./rxjs-imports";
import {
  addLinksToState,
  ensureResourceTypeInState,
  removeResourceFromState,
  setIsInvalidatingForExistingResource,
  updateOrInsertResourcesIntoState
} from "./state-mutation";
import { apiRequest } from "./utils";

const imm = require("object-path-immutable");

const {
  API_SET_ENDPOINT_HOST,
  API_SET_ENDPOINT_PATH,
  API_SET_HEADERS,
  API_SET_HEADER,
  API_HYDRATE,
  API_WILL_CREATE,
  API_CREATED,
  API_CREATE_FAILED,
  API_WILL_READ,
  API_READ,
  API_READ_FAILED,
  API_WILL_UPDATE,
  API_UPDATED,
  API_UPDATE_FAILED,
  API_WILL_DELETE,
  API_DELETED,
  API_DELETE_FAILED
} = actionTypes;

// Export __assign for unit tests
export { __assign };

// Resource isInvalidating values
export const IS_DELETING = "IS_DELETING";
export const IS_UPDATING = "IS_UPDATING";

// Action creators
export const setEndpointHost = createAction(API_SET_ENDPOINT_HOST);
export const setEndpointPath = createAction(API_SET_ENDPOINT_PATH);
export const setHeaders = createAction(API_SET_HEADERS);
export const setHeader = createAction(API_SET_HEADER);

export const hydrateStore = createAction(API_HYDRATE);

const apiWillCreate = createAction(API_WILL_CREATE);
const apiCreated = createAction(API_CREATED);
const apiCreateFailed = createAction(API_CREATE_FAILED);

const apiWillRead = createAction(API_WILL_READ);
const apiRead = createAction(API_READ);
const apiReadFailed = createAction(API_READ_FAILED);

const apiWillUpdate = createAction(API_WILL_UPDATE);
const apiUpdated = createAction(API_UPDATED);
const apiUpdateFailed = createAction(API_UPDATE_FAILED);

const apiWillDelete = createAction(API_WILL_DELETE);
const apiDeleted = createAction(API_DELETED);
const apiDeleteFailed = createAction(API_DELETE_FAILED);

// Actions
export const setAccessToken = at => {
  return dispatch => {
    dispatch(setHeader({ Authorization: `Bearer ${at}` }));
  };
};

export const createResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillCreate(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}`;

    return apiRequest(endpoint, {
      headers,
      method: "POST",
      withCredentials: true,
      body: JSON.stringify({
        data: resource
      })
    })
      .map(json => {
        dispatch(apiCreated(json));
        return json;
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiCreateFailed(err));
        return Observable.throw(err);
      });
  };
};

export const readEndpoint = (
  endpoint,
  {
    options = {
      indexLinks: undefined
    }
  } = {}
) => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillRead(endpoint));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const apiEndpoint = `${apiHost}${apiPath}/${endpoint}`;

    return apiRequest(`${apiEndpoint}`, {
      headers,
      withCredentials: true
    })
      .map(json => {
        dispatch(apiRead({ endpoint, options, ...json }));
        return json;
      })
      .catch(error => {
        const err = error;
        err.endpoint = endpoint;

        dispatch(apiReadFailed(err));
        return Observable.throw(err);
      });
  };
};

export const updateResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillUpdate(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}/${resource.id}`;

    return apiRequest(endpoint, {
      headers,
      method: "PATCH",
      withCredentials: true,
      body: JSON.stringify({
        data: resource
      })
    })
      .map(json => {
        dispatch(apiUpdated(json));
        return json;
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiUpdateFailed(err));
        return Observable.throw(err);
      });
  };
};

export const deleteResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillDelete(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}/${resource.id}`;

    return apiRequest(endpoint, {
      headers,
      method: "DELETE",
      withCredentials: true
    })
      .do(() => {
        dispatch(apiDeleted(resource));
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiDeleteFailed(err));
        return Observable.throw(err);
      });
  };
};

export const requireResource = (resourceType, endpoint = resourceType) => {
  return (dispatch, getState): Observable<any> => {
    const { api } = getState();
    if (api.hasOwnProperty(resourceType)) {
      return Observable.of();
    }

    return dispatch(readEndpoint(endpoint));
  };
};

// Reducers
export const reducer = handleActions(
  {
    [API_SET_HEADERS]: (state, { payload: headers }) => {
      return imm(state)
        .set(["endpoint", "headers"], headers)
        .value();
    },

    [API_SET_HEADER]: (state, { payload: header }) => {
      const newState = imm(state);
      Object.keys(header).forEach(key => {
        newState.set(["endpoint", "headers", key], header[key]);
      });

      return newState.value();
    },

    [API_SET_ENDPOINT_HOST]: (state, { payload: host }) => {
      return imm(state)
        .set(["endpoint", "host"], host)
        .value();
    },

    [API_SET_ENDPOINT_PATH]: (state, { payload: path }) => {
      return imm(state)
        .set(["endpoint", "path"], path)
        .value();
    },

    [API_HYDRATE]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState).value();
    },

    [API_WILL_CREATE]: state => {
      return imm(state)
        .set(["isCreating"], state.isCreating + 1)
        .value();
    },

    [API_CREATED]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState)
        .set("isCreating", state.isCreating - 1)
        .value();
    },

    [API_CREATE_FAILED]: state => {
      return imm(state)
        .set(["isCreating"], state.isCreating - 1)
        .value();
    },

    [API_WILL_READ]: state => {
      return imm(state)
        .set(["isReading"], state.isReading + 1)
        .value();
    },

    [API_READ]: (state, { payload }) => {
      const resources = (Array.isArray(payload.data)
        ? payload.data
        : [payload.data]
      ).concat(payload.included || []);

      const newState = updateOrInsertResourcesIntoState(state, resources);
      const finalState = addLinksToState(
        newState,
        payload.links,
        payload.options
      );

      return imm(finalState)
        .set("isReading", state.isReading - 1)
        .value();
    },

    [API_READ_FAILED]: state => {
      return imm(state)
        .set(["isReading"], state.isReading - 1)
        .value();
    },

    [API_WILL_UPDATE]: (state, { payload: resource }) => {
      const { type, id } = resource;

      const newState = ensureResourceTypeInState(state, type);

      return setIsInvalidatingForExistingResource(
        newState,
        { type, id },
        IS_UPDATING
      )
        .set("isUpdating", state.isUpdating + 1)
        .value();
    },

    [API_UPDATED]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState)
        .set("isUpdating", state.isUpdating - 1)
        .value();
    },

    [API_UPDATE_FAILED]: (state, { payload: { resource } }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_UPDATING
      )
        .set("isUpdating", state.isUpdating + 1)
        .value();
    },

    [API_WILL_DELETE]: (state, { payload: resource }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_DELETING
      )
        .set("isDeleting", state.isDeleting + 1)
        .value();
    },

    [API_DELETED]: (state, { payload: resource }) => {
      return removeResourceFromState(state, resource)
        .set("isDeleting", state.isDeleting - 1)
        .value();
    },

    [API_DELETE_FAILED]: (state, { payload: { resource } }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_DELETING
      )
        .set("isDeleting", state.isDeleting + 1)
        .value();
    }
  },
  {
    isCreating: 0,
    isReading: 0,
    isUpdating: 0,
    isDeleting: 0,
    endpoint: {
      host: null,
      path: null,
      headers: {
        "Content-Type": "application/vnd.api+json",
        Accept: "application/vnd.api+json"
      }
    }
  }
);

==> observable-redux-json-api-0.0.10.tgz: package/src/jsonapi.ts <==
import { __assign } from "./assign";

import { createAction, handleActions } from "redux-actions";
import actionTypes from "./constants";
import { Observable } from "./rxjs-imports";
import {
  addLinksToState,
  ensureResourceTypeInState,
  removeResourceFromState,
  setIsInvalidatingForExistingResource,
  updateOrInsertResourcesIntoState
} from "./state-mutation";
import { apiRequest } from "./utils";

const imm = require("object-path-immutable");

const {
  API_SET_ENDPOINT_HOST,
  API_SET_ENDPOINT_PATH,
  API_SET_HEADERS,
  API_SET_HEADER,
  API_HYDRATE,
  API_WILL_CREATE,
  API_CREATED,
  API_CREATE_FAILED,
  API_WILL_READ,
  API_READ,
  API_READ_FAILED,
  API_WILL_UPDATE,
  API_UPDATED,
  API_UPDATE_FAILED,
  API_WILL_DELETE,
  API_DELETED,
  API_DELETE_FAILED
} = actionTypes;

export { __assign }; // workaround to stop TS removing __assign import as unused

// Resource isInvalidating values
export const IS_DELETING = "IS_DELETING";
export const IS_UPDATING = "IS_UPDATING";

// Action creators
export const setEndpointHost = createAction(API_SET_ENDPOINT_HOST);
export const setEndpointPath = createAction(API_SET_ENDPOINT_PATH);
export const setHeaders = createAction(API_SET_HEADERS);
export const setHeader = createAction(API_SET_HEADER);

export const hydrateStore = createAction(API_HYDRATE);

const apiWillCreate = createAction(API_WILL_CREATE);
const apiCreated = createAction(API_CREATED);
const apiCreateFailed = createAction(API_CREATE_FAILED);

const apiWillRead = createAction(API_WILL_READ);
const apiRead = createAction(API_READ);
const apiReadFailed = createAction(API_READ_FAILED);

const apiWillUpdate = createAction(API_WILL_UPDATE);
const apiUpdated = createAction(API_UPDATED);
const apiUpdateFailed = createAction(API_UPDATE_FAILED);

const apiWillDelete = createAction(API_WILL_DELETE);
const apiDeleted = createAction(API_DELETED);
const apiDeleteFailed = createAction(API_DELETE_FAILED);

// Actions
export const setAccessToken = at => {
  return dispatch => {
    dispatch(setHeader({ Authorization: `Bearer ${at}` }));
  };
};

export const createResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillCreate(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}`;

    return apiRequest(endpoint, {
      headers,
      method: "POST",
      withCredentials: true,
      body: JSON.stringify({
        data: resource
      })
    })
      .map(json => {
        dispatch(apiCreated(json));
        return json;
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiCreateFailed(err));
        return Observable.throw(err);
      });
  };
};

export const readEndpoint = (
  endpoint,
  {
    options = {
      indexLinks: undefined
    }
  } = {}
) => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillRead(endpoint));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const apiEndpoint = `${apiHost}${apiPath}/${endpoint}`;

    return apiRequest(`${apiEndpoint}`, {
      headers,
      withCredentials: true
    })
      .map(json => {
        dispatch(apiRead({ endpoint, options, ...json }));
        return json;
      })
      .catch(error => {
        const err = error;
        err.endpoint = endpoint;

        dispatch(apiReadFailed(err));
        return Observable.throw(err);
      });
  };
};

export const updateResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillUpdate(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}/${resource.id}`;

    return apiRequest(endpoint, {
      headers,
      method: "PATCH",
      withCredentials: true,
      body: JSON.stringify({
        data: resource
      })
    })
      .map(json => {
        dispatch(apiUpdated(json));
        return json;
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiUpdateFailed(err));
        return Observable.throw(err);
      });
  };
};

export const deleteResource = resource => {
  return (dispatch, getState): Observable<any> => {
    dispatch(apiWillDelete(resource));

    const { host: apiHost, path: apiPath, headers } = getState().api.endpoint;
    const endpoint = `${apiHost}${apiPath}/${resource.type}/${resource.id}`;

    return apiRequest(endpoint, {
      headers,
      method: "DELETE",
      withCredentials: true
    })
      .do(() => {
        dispatch(apiDeleted(resource));
      })
      .catch(error => {
        const err = error;
        err.resource = resource;

        dispatch(apiDeleteFailed(err));
        return Observable.throw(err);
      });
  };
};

export const requireResource = (resourceType, endpoint = resourceType) => {
  return (dispatch, getState): Observable<any> => {
    const { api } = getState();
    if (api.hasOwnProperty(resourceType)) {
      return Observable.of();
    }

    return dispatch(readEndpoint(endpoint));
  };
};

// Reducers
export const reducer = handleActions(
  {
    [API_SET_HEADERS]: (state, { payload: headers }) => {
      return imm(state)
        .set(["endpoint", "headers"], headers)
        .value();
    },

    [API_SET_HEADER]: (state, { payload: header }) => {
      const newState = imm(state);
      Object.keys(header).forEach(key => {
        newState.set(["endpoint", "headers", key], header[key]);
      });

      return newState.value();
    },

    [API_SET_ENDPOINT_HOST]: (state, { payload: host }) => {
      return imm(state)
        .set(["endpoint", "host"], host)
        .value();
    },

    [API_SET_ENDPOINT_PATH]: (state, { payload: path }) => {
      return imm(state)
        .set(["endpoint", "path"], path)
        .value();
    },

    [API_HYDRATE]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState).value();
    },

    [API_WILL_CREATE]: state => {
      return imm(state)
        .set(["isCreating"], state.isCreating + 1)
        .value();
    },

    [API_CREATED]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState)
        .set("isCreating", state.isCreating - 1)
        .value();
    },

    [API_CREATE_FAILED]: state => {
      return imm(state)
        .set(["isCreating"], state.isCreating - 1)
        .value();
    },

    [API_WILL_READ]: state => {
      return imm(state)
        .set(["isReading"], state.isReading + 1)
        .value();
    },

    [API_READ]: (state, { payload }) => {
      const resources = (Array.isArray(payload.data)
        ? payload.data
        : [payload.data]
      ).concat(payload.included || []);

      const newState = updateOrInsertResourcesIntoState(state, resources);
      const finalState = addLinksToState(
        newState,
        payload.links,
        payload.options
      );

      return imm(finalState)
        .set("isReading", state.isReading - 1)
        .value();
    },

    [API_READ_FAILED]: state => {
      return imm(state)
        .set(["isReading"], state.isReading - 1)
        .value();
    },

    [API_WILL_UPDATE]: (state, { payload: resource }) => {
      const { type, id } = resource;

      const newState = ensureResourceTypeInState(state, type);

      return setIsInvalidatingForExistingResource(
        newState,
        { type, id },
        IS_UPDATING
      )
        .set("isUpdating", state.isUpdating + 1)
        .value();
    },

    [API_UPDATED]: (state, { payload: resources }) => {
      const entities = Array.isArray(resources.data)
        ? resources.data
        : [resources.data];

      const newState = updateOrInsertResourcesIntoState(
        state,
        entities.concat(resources.included || [])
      );

      return imm(newState)
        .set("isUpdating", state.isUpdating - 1)
        .value();
    },

    [API_UPDATE_FAILED]: (state, { payload: { resource } }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_UPDATING
      )
        .set("isUpdating", state.isUpdating + 1)
        .value();
    },

    [API_WILL_DELETE]: (state, { payload: resource }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_DELETING
      )
        .set("isDeleting", state.isDeleting + 1)
        .value();
    },

    [API_DELETED]: (state, { payload: resource }) => {
      return removeResourceFromState(state, resource)
        .set("isDeleting", state.isDeleting - 1)
        .value();
    },

    [API_DELETE_FAILED]: (state, { payload: { resource } }) => {
      const { type, id } = resource;

      return setIsInvalidatingForExistingResource(
        state,
        { type, id },
        IS_DELETING
      )
        .set("isDeleting", state.isDeleting + 1)
        .value();
    }
  },
  {
    isCreating: 0,
    isReading: 0,
    isUpdating: 0,
    isDeleting: 0,
    endpoint: {
      host: null,
      path: null,
      headers: {
        "Content-Type": "application/vnd.api+json",
        Accept: "application/vnd.api+json"
      }
    }
  }
);
